// hlavickove subory *******************************************************
// systemove funkcie
#include <stdio.h>
#include <string.h>
#include <math.h>
// GL a GLU kniznica
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glext.h>
// funkcie SDL kniznice
#include <SDL/SDL.h>

// .h subor
#include "mOGL.h"

// objekt torusu
Torus m_torus(16);

// konstruktor *************************************************************
mOGL::mOGL()
{
 // rozmery okna
 ViewPort[0]=0; ViewPort[1]=0; ViewPort[2]=0; ViewPort[3]=0;

 Cnum=0; // cislo pouzitej OpenGL kamery

 StartInfoPos=10.0f; // pozicia vypisu 1. textu pocas startu aplikacie

 t_a=0; t_b=0; t_c=0; // pomocne premenne

 lighting_onoff=0;  // osvetlenie
 help_onoff=0;      // napoveda
 scene_draw_type=0; // standardny pohlad na scenu
 pause_motion=0;    // casovo zavisly pohyb
 normals_onoff=0;   // vypnutie vykreslovania normal

 mtime=0.0f; // cas

 SOvertex=NULL; SOnormal=NULL; SOtangent=NULL; SObinormal=NULL; SOuv=NULL;
 SOvertex_num=0;

 sc=0.25f; // skalovaci faktor
}

// destruktor **************************************************************
mOGL::~mOGL()
{
 // uvolnenie alokovanej pamate
 if (SOvertex!=NULL) delete SOvertex;
 if (SOnormal!=NULL) delete SOnormal;
 if (SOtangent!=NULL) delete SOtangent;
 if (SObinormal!=NULL) delete SObinormal;
 if (SOuv!=NULL) delete SOuv;
}

// vypis info pocas startu aplikacie ***************************************
void mOGL::PrintStartInfo(char *infotext)
{
 ogl_Text.OpenDrawString();
 ogl_Text.DrawString(10.0f,StartInfoPos,infotext);
 ogl_Text.CloseDrawString();

 // vymena zobrazovacich zasobnikov, zobrazenie sceny
 SDL_GL_SwapBuffers();

 // dalsi text bude vypisany na novu poziciu
 StartInfoPos+=20.0f;
 // pauza aby bol text aspon chvilku viditelny
 SDL_Delay(100);
}

// inicializacia OpenGL - nastavenie parametrov pre tvorbu OpenGL sceny ****
bool mOGL::gl_initialize(void)
{
 // nastavenie 8-mich vseobecnych materialov
 for (t_a=0;t_a<8;t_a++) {
  ogl_Material[t_a].amb[0]=0.1f; ogl_Material[t_a].amb[1]=0.1f;
  ogl_Material[t_a].amb[2]=0.1f; ogl_Material[t_a].amb[3]=1.0f;
  ogl_Material[t_a].dif[0]=0.8f; ogl_Material[t_a].dif[1]=0.8f;
  ogl_Material[t_a].dif[2]=0.8f; ogl_Material[t_a].dif[3]=1.0f;
  ogl_Material[t_a].spe[0]=1.0f; ogl_Material[t_a].spe[1]=1.0f;
  ogl_Material[t_a].spe[2]=1.0f; ogl_Material[t_a].spe[3]=1.0f;
  ogl_Material[t_a].amb_dif[0]=0.25f; ogl_Material[t_a].amb_dif[1]=0.25f;
  ogl_Material[t_a].amb_dif[2]=0.25f; ogl_Material[t_a].amb_dif[3]=1.0f;
  ogl_Material[t_a].emi[0]=0.0f; ogl_Material[t_a].emi[1]=0.0f;
  ogl_Material[t_a].emi[2]=0.0f; ogl_Material[t_a].emi[3]=1.0f;
  ogl_Material[t_a].shi=128.0f;
  ogl_Material[t_a].transp=0.0f;
  ogl_Material[t_a].sillum[0]=0.0f; ogl_Material[t_a].sillum[1]=0.0f;
  ogl_Material[t_a].sillum[2]=0.0f; ogl_Material[t_a].sillum[3]=1.0f;
 }
 // nastavenie 8-mich vseobecnych svetiel
 for (t_a=0;t_a<8;t_a++) {
  ogl_Light[t_a].amb[0]=0.1f; ogl_Light[t_a].amb[1]=0.1f;
  ogl_Light[t_a].amb[2]=0.1f; ogl_Light[t_a].amb[3]=1.0f;
  ogl_Light[t_a].dif[0]=0.9f; ogl_Light[t_a].dif[1]=0.9f;
  ogl_Light[t_a].dif[2]=0.9f; ogl_Light[t_a].dif[3]=1.0f;
  ogl_Light[t_a].spe[0]=0.9f; ogl_Light[t_a].spe[1]=0.9f;
  ogl_Light[t_a].spe[2]=0.9f; ogl_Light[t_a].spe[3]=1.0f;
  ogl_Light[t_a].pos[0]=-10.0f; ogl_Light[t_a].pos[1]=10.0f;
  ogl_Light[t_a].pos[2]=10.0f; ogl_Light[t_a].pos[3]=1.0f;
  // pos[3]=1.0f - vsesmerove svetlo, 0.0f - smerove svetlo
  ogl_Light[t_a].spot[0]=0.0f;
  ogl_Light[t_a].spot[1]=0.0f;
  ogl_Light[t_a].spot[2]=-1.0f;
 }

 // STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ STAVOVY STROJ --

 // nastavenie OpenGL zasobnika farieb a hlbky *****************************
 glClearColor(0.0f,0.0f,0.0f,0.5f); // zasobnik farieb bude vymazavany ciernou farbou
 glClearDepth(1.0f);                // zasobnik hlbky bude vymazavany nastavenim na 1.0f
 //glClearStencil(0);               // stencil zasobnik bude vymazavany nastavenim na 0

 glDepthFunc(GL_LEQUAL);  // typ porovnavacej funkcie pre urcenie Z-vzdialenosti
 glEnable(GL_DEPTH_TEST); // zapnutie testovania hlbky

 glDisable(GL_CULL_FACE); // nastavenie poradia, v ktorom budu definovane vrcholy objektu
 glFrontFace(GL_CCW);     // reprezentovat privratene resp. odvratene plochy,
 glCullFace(GL_BACK);     // CCW - proti smeru hod. ruciciek, CW - v smere hod. ruciciek
 glEnable(GL_CULL_FACE);  // odvratene plochy sa nebudu spracovavat

 // nastavenie parametrov 1. OpenGL svetla
 glLightfv(GL_LIGHT0,GL_AMBIENT,ogl_Light[0].amb);
 glLightfv(GL_LIGHT0,GL_DIFFUSE,ogl_Light[0].dif);
 glLightfv(GL_LIGHT0,GL_SPECULAR,ogl_Light[0].spe);
 glLightfv(GL_LIGHT0,GL_POSITION,ogl_Light[0].pos);
 glEnable(GL_LIGHT0); // ON
 // nastavenie parametrov 2. OpenGL svetla
 glLightfv(GL_LIGHT1,GL_AMBIENT,ogl_Light[1].amb);
 glLightfv(GL_LIGHT1,GL_DIFFUSE,ogl_Light[1].dif);
 glLightfv(GL_LIGHT1,GL_SPECULAR,ogl_Light[1].spe);
 glLightfv(GL_LIGHT1,GL_POSITION,ogl_Light[1].pos);
 //glEnable(GL_LIGHT1); // OFF
 if (lighting_onoff) glEnable(GL_LIGHTING);
 else glDisable(GL_LIGHTING);
 // svetlo sa pocita len pre privratene strany
 glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,0);
 // svetelne vypocty su lokalne
 glLightModelf(GL_LIGHT_MODEL_LOCAL_VIEWER,1.0f);

 // vseobecny material objektov
 glMaterialfv(GL_FRONT,GL_AMBIENT,ogl_Material[0].amb);
 glMaterialfv(GL_FRONT,GL_DIFFUSE,ogl_Material[0].dif);
 glMaterialfv(GL_FRONT,GL_SPECULAR,ogl_Material[0].spe);
 glMaterialfv(GL_FRONT,GL_EMISSION,ogl_Material[0].emi);
 glMaterialf(GL_FRONT,GL_SHININESS,ogl_Material[0].shi);
 // farba bude mat vplyv iba na 'diffuse' komponent materialu
 glColorMaterial(GL_FRONT,GL_DIFFUSE);
 // pre Gourandove svetelne vypocty bude pouzity material priradeny objektu,
 // ktory bude ale stale ovplyvneny farbou zadavanou prikazom glColor
 glEnable(GL_COLOR_MATERIAL);
 // zapnutie gourand-jemneho tienovania
 glShadeModel(GL_SMOOTH);
 // trojuholniky budu vyplnene
 glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);

 // realnejsie perspektivne vypocty a korekcie
 glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);

 // zapnutie modulacie textur osvetlenim
 // vysledna farba bodu = farba z textury + osvetlenie
 glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);

 // INICIALIZACIA SCENY INICIALIZACIA SCENY INICIALIZACIA SCENY ------------

 fprintf(stderr,"----------------------------------------------------------\n");

 // vymazanie zasobnika farieb, hlbky a stencil zasobnika
 glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

 // inicializacia vypisovania OpenGL textu
 if (ogl_Text.TextInitialize()) {
  fprintf(stderr,"Chyba inicializacie fontu!\n"); return(1);
 }
 // potvrdenie startu OpenGL
 PrintStartInfo("Nastavenie OpenGL stavoveho stroja...");
 PrintStartInfo("OK");

 // zameriavaci kriz
 PrintStartInfo("Nahratie textury zameriavacieho kriza...");
 if (ogl_Texture.LoadCrosshairTexture("data/gfx/crosshair0.bmp")) {
  fprintf(stderr,"Chyba nahratia textury zameriavacieho kriza!\n"); return(1);
 }
 PrintStartInfo("OK");

 // textury
 PrintStartInfo("Nahratie textur...");
 if (ogl_Texture.LoadTexture(0,"data/textures/stone.tga",T_TRILINEAR|T_MIPMAP)) {
  fprintf(stderr,"Chyba nahratia textury!\n"); return(1);
 }
 if (ogl_Texture.LoadTexture(1,"data/textures/attenuate2d.tga",T_CLAMP)) {
  fprintf(stderr,"Chyba nahratia textury!\n"); return(1);
 }
 if (ogl_Texture.Load1DTexture(2,"data/textures/attenuate1d.tga",T_CLAMP)) {
  fprintf(stderr,"Chyba nahratia textury!\n"); return(1);
 }
 if (ogl_Texture.LoadTexture(3,"data/gfx/flare.tga",T_DEFAULT)) {
  fprintf(stderr,"Chyba nahratia textury!\n"); return(1);
 }
 PrintStartInfo("OK");

 PrintStartInfo("Test rozsireni...");
 // 1. krok - test ci karta podporuje rozsirenia............................
 torus_fp.CheckExtension("GL_ARB_multitexture");
 torus_fp.CheckExtension("GL_ARB_vertex_program");
 torus_fp.CheckExtension("GL_ARB_fragment_program");
 PrintStartInfo("OK");

 PrintStartInfo("Priprava osvetleneho objektu...");
 // 2. krok - vytovrenie objektu, ktory bude osvetelny svetlami
 // bude sa jednat o podlahu a torus .......................................
 SOvertex_num=6;
 SOvertex=new tVec3[SOvertex_num];   // pole vrcholov
 SOnormal=new tVec3[SOvertex_num];   // pole normal
 SOtangent=new tVec3[SOvertex_num];  // pole tangent-vektorov - v smere U texturovacich suradnic
 SObinormal=new tVec3[SOvertex_num]; // pole binormal-vektorov - v smere V texturovacich suradnic
 SOuv=new tVec2[SOvertex_num];       // pole UV texturovacich suradnic
 // podlaha
 SOvertex[0].x=-10.0f; SOvertex[0].y=0.0f; SOvertex[0].z=10.0f;
 SOvertex[1].x=10.0f;  SOvertex[1].y=0.0f; SOvertex[1].z=10.0f;
 SOvertex[2].x=-10.0f; SOvertex[2].y=0.0f; SOvertex[2].z=-10.0f;
 SOvertex[3].x=10.0f;  SOvertex[3].y=0.0f; SOvertex[3].z=10.0f;
 SOvertex[4].x=10.0f;  SOvertex[4].y=0.0f; SOvertex[4].z=-10.0f;
 SOvertex[5].x=-10.0f; SOvertex[5].y=0.0f; SOvertex[5].z=-10.0f;

 SOnormal[0].x=0.0f; SOnormal[0].y=1.0f; SOnormal[0].z=0.0f;
 SOnormal[1].x=0.0f; SOnormal[1].y=1.0f; SOnormal[1].z=0.0f;
 SOnormal[2].x=0.0f; SOnormal[2].y=1.0f; SOnormal[2].z=0.0f;
 SOnormal[3].x=0.0f; SOnormal[3].y=1.0f; SOnormal[3].z=0.0f;
 SOnormal[4].x=0.0f; SOnormal[4].y=1.0f; SOnormal[4].z=0.0f;
 SOnormal[5].x=0.0f; SOnormal[5].y=1.0f; SOnormal[5].z=0.0f;

 SOtangent[0].x=1.0f; SOtangent[0].y=0.0f; SOtangent[0].z=0.0f;
 SOtangent[1].x=1.0f; SOtangent[1].y=0.0f; SOtangent[1].z=0.0f;
 SOtangent[2].x=1.0f; SOtangent[2].y=0.0f; SOtangent[2].z=0.0f;
 SOtangent[3].x=1.0f; SOtangent[3].y=0.0f; SOtangent[3].z=0.0f;
 SOtangent[4].x=1.0f; SOtangent[4].y=0.0f; SOtangent[4].z=0.0f;
 SOtangent[5].x=1.0f; SOtangent[5].y=0.0f; SOtangent[5].z=0.0f;

 SObinormal[0].x=0.0f; SObinormal[0].y=0.0f; SObinormal[0].z=-1.0f;
 SObinormal[1].x=0.0f; SObinormal[1].y=0.0f; SObinormal[1].z=-1.0f;
 SObinormal[2].x=0.0f; SObinormal[2].y=0.0f; SObinormal[2].z=-1.0f;
 SObinormal[3].x=0.0f; SObinormal[3].y=0.0f; SObinormal[3].z=-1.0f;
 SObinormal[4].x=0.0f; SObinormal[4].y=0.0f; SObinormal[4].z=-1.0f;
 SObinormal[5].x=0.0f; SObinormal[5].y=0.0f; SObinormal[5].z=-1.0f;

 SOuv[0].x=0.0f; SOuv[0].y=0.0f;
 SOuv[1].x=2.0f; SOuv[1].y=0.0f;
 SOuv[2].x=0.0f; SOuv[2].y=2.0f;
 SOuv[3].x=2.0f; SOuv[3].y=0.0f;
 SOuv[4].x=2.0f; SOuv[4].y=2.0f;
 SOuv[5].x=0.0f; SOuv[5].y=2.0f;

 // objekty budu ulozene v display listoch
 SOdl_id=glGenLists(2);
 // klasicke vykreslenie objektu (podlaha) - DL 0
 glNewList(SOdl_id,GL_COMPILE);
  glEnableClientState(GL_VERTEX_ARRAY);
  glEnableClientState(GL_NORMAL_ARRAY);
  glEnableClientState(GL_TEXTURE_COORD_ARRAY);
  glVertexPointer(3,GL_FLOAT,0,SOvertex);
  glNormalPointer(GL_FLOAT,0,SOnormal);
  glTexCoordPointer(2,GL_FLOAT,0,SOuv);
  glDrawArrays(GL_TRIANGLES,0,6);
  glDisableClientState(GL_TEXTURE_COORD_ARRAY);
  glDisableClientState(GL_NORMAL_ARRAY);
  glDisableClientState(GL_VERTEX_ARRAY);
 glEndList();
 // klasicke vykreslenie objektu (torus) - DL 1
 glNewList(SOdl_id+1,GL_COMPILE);
  glEnableClientState(GL_VERTEX_ARRAY);
  glEnableClientState(GL_NORMAL_ARRAY);
  glEnableClientState(GL_TEXTURE_COORD_ARRAY);
  glVertexPointer(3,GL_FLOAT,sizeof(TorusVertex),&m_torus.vertices[0].position);
  glNormalPointer(GL_FLOAT,sizeof(TorusVertex),&m_torus.vertices[0].normal);
  glTexCoordPointer(2,GL_FLOAT,sizeof(TorusVertex),&m_torus.vertices[0].s);
  glDrawElements(GL_TRIANGLES,static_cast<GLsizei>(m_torus.indices.size()),GL_UNSIGNED_INT,&m_torus.indices[0]);
  glDisableClientState(GL_TEXTURE_COORD_ARRAY);
  glDisableClientState(GL_NORMAL_ARRAY);
  glDisableClientState(GL_VERTEX_ARRAY);
 glEndList();
 PrintStartInfo("OK");

 PrintStartInfo("Nahratie shader programov...");
 // 3. krok - nahratie Vertex a Fragment Programu pre LightShader ..........
 //torus_fp.PRGload("data/shaders/torus.fp");
 point_vp.PRGload("data/shaders/point.vp");
 //default_vp.PRGload("data/shaders/default.vp");
 PrintStartInfo("OK");

 // 4. krok - nastavenie pociatocnej polohy a farby svetiel ................
 light1.x=0.0f; light1.y=0.0f; light1.z=0.0f; light1.w=1.0f;
 light2.x=0.0f; light2.y=0.0f; light2.z=0.0f; light2.w=1.0f;
 light3.x=0.0f; light3.y=0.0f; light3.z=0.0f; light3.w=1.0f;
 lcolor1.x=1.0f; lcolor1.y=0.0f; lcolor1.z=0.0f;
 lcolor2.x=0.0f; lcolor2.y=1.0f; lcolor2.z=0.0f;
 lcolor3.x=0.0f; lcolor3.y=0.0f; lcolor3.z=1.0f;

 // startovaci cas
 mtime=0.0f;

 // priprava pouzitia rozsireni
 torus_fp.Ext_init();

 // koniec pociatocnych nastaveni, zaciatok interakcie pozorovatela
 PrintStartInfo("START");

 // nastavenie farby a velkosti neskor vypisovaneho textu
 ogl_Text.SetTextColor(1.0f,1.0f,1.0f);
 ogl_Text.SetTextSize(10);

 // nastavenie startovacej farby - biela
 glColor4f(1.0f,1.0f,1.0f,1.0f);

 return(0);
}

// zapnutie Register kombinerov ********************************************
void mOGL::RegCombinersON(float *color)
{
 // 1. texturovacia jednotka - 2D utlm
 glActiveTextureARB(GL_TEXTURE0_ARB);
 glEnable(GL_TEXTURE_2D);
 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[1]);
 // 2. texturovacia jednotka - 1D utlm
 glActiveTextureARB(GL_TEXTURE1_ARB);
 glEnable(GL_TEXTURE_1D);
 glBindTexture(GL_TEXTURE_1D,ogl_Texture.texture_id[2]);

 // poziadanie o 1 vseobecny Register kombiner
 glCombinerParameteriNV(GL_NUM_GENERAL_COMBINERS_NV,1);
 // nastavenie registra 'constant color 0' podla farby svetla
 glCombinerParameterfvNV(GL_CONSTANT_COLOR0_NV,color);

 // A=2D utlm, B=1, C=1D utlm, D=1
 glCombinerInputNV(GL_COMBINER0_NV,GL_RGB,GL_VARIABLE_A_NV,GL_TEXTURE0_ARB,GL_UNSIGNED_IDENTITY_NV,GL_RGB);
 glCombinerInputNV(GL_COMBINER0_NV,GL_RGB,GL_VARIABLE_B_NV,GL_ZERO,GL_UNSIGNED_INVERT_NV,GL_RGB);
 glCombinerInputNV(GL_COMBINER0_NV,GL_RGB,GL_VARIABLE_C_NV,GL_TEXTURE1_ARB,GL_UNSIGNED_IDENTITY_NV,GL_RGB);
 glCombinerInputNV(GL_COMBINER0_NV,GL_RGB,GL_VARIABLE_D_NV,GL_ZERO,GL_UNSIGNED_INVERT_NV,GL_RGB);

 // vystup kombinera bude:
 // AB=0, CD=0, spare0=sum (AB+CD vystup), scale=0, bias=0, ABdot=0, CDdot=0, muxSum=0
 glCombinerOutputNV(GL_COMBINER0_NV,GL_RGB,GL_DISCARD_NV,GL_DISCARD_NV,GL_SPARE0_NV,GL_NONE,GL_NONE,GL_FALSE,GL_FALSE,GL_FALSE);

 // A=sum, B=0, C=farba svetla, D=0, G=0
 glFinalCombinerInputNV(GL_VARIABLE_A_NV,GL_SPARE0_NV,GL_UNSIGNED_IDENTITY_NV,GL_RGB);
 glFinalCombinerInputNV(GL_VARIABLE_B_NV,GL_ZERO,GL_UNSIGNED_IDENTITY_NV,GL_RGB);
 glFinalCombinerInputNV(GL_VARIABLE_C_NV,GL_CONSTANT_COLOR0_NV,GL_UNSIGNED_IDENTITY_NV,GL_RGB);
 glFinalCombinerInputNV(GL_VARIABLE_D_NV,GL_ZERO,GL_UNSIGNED_IDENTITY_NV,GL_RGB);
 glFinalCombinerInputNV(GL_VARIABLE_G_NV,GL_ZERO,GL_UNSIGNED_INVERT_NV,GL_ALPHA);
 // vystup finalneho kombinera bude:
 // AB+(1-A)C+D=fragment RGB, G=fragment alpha

 // zapnutie pouzivania Register kombinerov
 glEnable(GL_REGISTER_COMBINERS_NV);
}

// vypnutie Register kombinerov ********************************************
void mOGL::RegCombinersOFF(void)
{
 // vypnutie pouzivania Register kombinerov
 glDisable(GL_REGISTER_COMBINERS_NV);
 // vypnutie 2. texturovacej jednotky
 glActiveTextureARB(GL_TEXTURE1_ARB);
 glDisable(GL_TEXTURE_1D);
 // vypnutie 1. texturovacej jednotky
 glActiveTextureARB(GL_TEXTURE0_ARB);
 glDisable(GL_TEXTURE_2D);
}

// vykreslenie OpenGL sceny ************************************************
int mOGL::gl_draw(void)
{
 // vymazanie zasobnika farieb a hlbky
 glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

 // zresetovanie modelovej matice
 glLoadIdentity();

 // TRANSFORMACIA SCENY PODLA NASTAVENI AKTUALNEJ KAMERY!
 if (scene_draw_type==0) glMultMatrixf(ogl_Camera[Cnum].matrix);
 else {
  glRotatef(45.0f,1.0f,0.0f,0.0f);  // zapnutie specialneho pohladu na scenu
  glRotatef(-30.0f,0.0f,1.0f,0.0f);
 }
 // TRANSFORMACIA SCENY PODLA NASTAVENI AKTUALNEJ KAMERY!
 if (scene_draw_type==0) glTranslatef(ogl_Camera[Cnum].posx,ogl_Camera[Cnum].posy,ogl_Camera[Cnum].posz);
 else glTranslatef(-7.5f,-10.0f,-15.0f); // v pripade specialneho pohladu na scenu

 // transformacia polohy svetla podla polohy kamery
 //ogl_Light[0].pos[0]=-ogl_Camera[0].posx;
 //ogl_Light[0].pos[1]=-ogl_Camera[0].posy;
 //ogl_Light[0].pos[2]=-ogl_Camera[0].posz;

 // umiestenenie svetla na preddefinovanu poziciu
 glLightfv(GL_LIGHT0,GL_POSITION,ogl_Light[0].pos);

 // prirastok uhlov natocenia svetiel
 angle1=mtime/1.0f;
 angle2=mtime/2.0f;
 angle3=mtime/4.0f;

 // vypocet polohy svetiel
 light1=tVec3( 4.0f*sin(angle1),3.5f+1.5f*sin(angle1*3.0f),4.0f*cos(angle1));
 light2=tVec3(-6.0f*sin(angle2),3.5f+1.5f*sin(angle2*3.0f),4.0f*cos(angle2));
 light3=tVec3( 8.0f*sin(angle3),3.5f+1.5f*sin(angle3*3.0f),4.0f*cos(angle3));

 // vykreslia sa objekty osvetlene troma svetlami
 point_vp.PRGenable(); point_vp.PRGbind();
 // 1. svetlo
 col[0]=lcolor1.x; col[1]=lcolor1.y; col[2]=lcolor1.z;
 RegCombinersON(col);
 glLightfv(GL_LIGHT0,GL_POSITION,light1);
 glCallList(SOdl_id+0); glCallList(SOdl_id+1);
 RegCombinersOFF();

 glEnable(GL_BLEND);
 glBlendFunc(GL_ONE,GL_ONE);
 // 2. svetlo
 col[0]=lcolor2.x; col[1]=lcolor2.y; col[2]=lcolor2.z;
 RegCombinersON(col);
 glLightfv(GL_LIGHT0,GL_POSITION,light2);
 glCallList(SOdl_id+0);glCallList(SOdl_id+1);
 RegCombinersOFF();
 // 3. svetlo
 col[0]=lcolor3.x; col[1]=lcolor3.y; col[2]=lcolor3.z;
 RegCombinersON(col);
 glLightfv(GL_LIGHT0,GL_POSITION,light3);
 glCallList(SOdl_id+0); glCallList(SOdl_id+1);
 RegCombinersOFF();

 glBlendFunc(GL_DST_COLOR,GL_ZERO);

 point_vp.PRGdisable();

 // vykreslia sa objekty pokryte klasickou texturou
 //default_vp.PRGenable(); default_vp.PRGbind();

 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[0]);
 glEnable(GL_TEXTURE_2D);
 glCallList(SOdl_id+0); glCallList(SOdl_id+1);
 glDisable(GL_TEXTURE_2D);

 //default_vp.PRGdisable();

 glDisable(GL_BLEND);

 // vykreslia sa objekty svetiel
 glPushAttrib(GL_ALL_ATTRIB_BITS);

 glBlendFunc(GL_ONE,GL_ONE_MINUS_SRC_COLOR);
 glEnable(GL_BLEND);

 glBindTexture(GL_TEXTURE_2D,ogl_Texture.texture_id[3]);
 glEnable(GL_TEXTURE_2D);

 glGetFloatv(GL_MODELVIEW_MATRIX,mv);
 rightv.x=mv.mat[0]; rightv.y=mv.mat[4]; rightv.z=mv.mat[8];
 upv.x=mv.mat[1]; upv.y=mv.mat[5]; upv.z=mv.mat[9];

 // 1. svetlo
 glPushMatrix();
 glTranslatef(light1.x,light1.y,light1.z);
 glColor3f(lcolor1.x,lcolor1.y,lcolor1.z);
 // obycajny stvoruholnik pokryty texturou, otoceny vzdy k pozorovatelovi
 glBegin(GL_QUADS);
  glTexCoord2f(0.0f,0.0f); glVertex3f((rightv.x+upv.x)*(-sc),(rightv.y+upv.y)*(-sc),(rightv.z+upv.z)*(-sc));
  glTexCoord2f(1.0f,0.0f); glVertex3f((rightv.x-upv.x)*sc,(rightv.y-upv.y)*sc,(rightv.z-upv.z)*sc);
  glTexCoord2f(1.0f,1.0f); glVertex3f((rightv.x+upv.x)*sc,(rightv.y+upv.y)*sc,(rightv.z+upv.z)*sc);
  glTexCoord2f(0.0f,1.0f); glVertex3f((rightv.x-upv.x)*(-sc),(rightv.y-upv.y)*(-sc),(rightv.z-upv.z)*(-sc));
 glEnd();

 glPopMatrix();

 // 2.svetlo
 glPushMatrix();
 glTranslatef(light2.x,light2.y,light2.z);
 glColor3f(lcolor2.x,lcolor2.y,lcolor2.z);
 // obycajny stvoruholnik pokryty texturou, otoceny vzdy k pozorovatelovi
 glBegin(GL_QUADS);
  glTexCoord2f(0.0f,0.0f); glVertex3f((rightv.x+upv.x)*(-sc),(rightv.y+upv.y)*(-sc),(rightv.z+upv.z)*(-sc));
  glTexCoord2f(1.0f,0.0f); glVertex3f((rightv.x-upv.x)*sc,(rightv.y-upv.y)*sc,(rightv.z-upv.z)*sc);
  glTexCoord2f(1.0f,1.0f); glVertex3f((rightv.x+upv.x)*sc,(rightv.y+upv.y)*sc,(rightv.z+upv.z)*sc);
  glTexCoord2f(0.0f,1.0f); glVertex3f((rightv.x-upv.x)*(-sc),(rightv.y-upv.y)*(-sc),(rightv.z-upv.z)*(-sc));
 glEnd();

 glPopMatrix();

 // 3. svetlo
 glPushMatrix();
 glTranslatef(light3.x,light3.y,light3.z);
 glColor3f(lcolor3.x,lcolor3.y,lcolor3.z);
 // obycajny stvoruholnik pokryty texturou, otoceny vzdy k pozorovatelovi
 glBegin(GL_QUADS);
  glTexCoord2f(0.0f,0.0f); glVertex3f((rightv.x+upv.x)*(-sc),(rightv.y+upv.y)*(-sc),(rightv.z+upv.z)*(-sc));
  glTexCoord2f(1.0f,0.0f); glVertex3f((rightv.x-upv.x)*sc,(rightv.y-upv.y)*sc,(rightv.z-upv.z)*sc);
  glTexCoord2f(1.0f,1.0f); glVertex3f((rightv.x+upv.x)*sc,(rightv.y+upv.y)*sc,(rightv.z+upv.z)*sc);
  glTexCoord2f(0.0f,1.0f); glVertex3f((rightv.x-upv.x)*(-sc),(rightv.y-upv.y)*(-sc),(rightv.z-upv.z)*(-sc));
 glEnd();

 glPopMatrix();

 glPopAttrib();

 // nastavenie startovacej farby - biela
 glColor4f(1.0f,1.0f,1.0f,1.0f);

 // vykreslenie zameriavacieho kriza
 ogl_Texture.DrawCrosshair(ogl_Camera[Cnum].scr_w,ogl_Camera[Cnum].scr_h);

 if (help_onoff) {
  // vypis textu - help
  ogl_Text.OpenDrawString();

  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-20.0f, "ESC ukoncenie aplikacie");

  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-60.0f, "MYS                rotacia v danom smere");
  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-80.0f, "SIPKY              posun dopredu,dozadu,vlavo,vpravo");
  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-100.0f,"SHIFT              zrychlenie posunu");

  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-140.0f,"R     reinicializacia pohladu kamery");
  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-160.0f,"M     vyplnene trojuholniky/iba hrany trojuholnikov");
  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-180.0f,"P     zapnutie/vypnutie casovo zavisleho pohybu objektov");

  ogl_Text.DrawString(10.0f,ogl_Camera[Cnum].scr_h-220.0f,"H     zapnutie/vypnutie zobrazenia helpu");

  ogl_Text.CloseDrawString();
 }
 else {
  // vypis textu - poloha kamery
  ogl_Text.OpenDrawString();

  sprintf(ogl_Text.numberbuf,"x:%6.2f",-ogl_Camera[Cnum].posx);
  ogl_Text.DrawNumber(10.0f,30.0f);
  sprintf(ogl_Text.numberbuf,"y:%6.2f",-ogl_Camera[Cnum].posy);
  ogl_Text.DrawNumber(10.0f,20.0f);
  sprintf(ogl_Text.numberbuf,"z:%6.2f",-ogl_Camera[Cnum].posz);
  ogl_Text.DrawNumber(10.0f,10.0f);

  // vypis textu - pocet obrazkov za sekundu
  sprintf(ogl_Text.numberbuf,"%i fps",(GLuint) ogl_Camera[Cnum].ogl_fps);
  //sprintf(ogl_Text.numberbuf,"%f fps",ogl_Camera[Cnum].GetCameraTime()*1000.0f);
  ogl_Text.DrawNumber(10.0f,ogl_Camera[Cnum].scr_h-20.0f);

  ogl_Text.CloseDrawString();
 }

 return(0);
}

// casovo zavisla uprava objektov, resp. sceny *****************************
int mOGL::gl_iddle(void)
{
 if (pause_motion==0) {
  mtime+=ogl_Camera[Cnum].GetCameraTime();
 }

 return(0);
}

// zmena sposobu zobrazenia objektov ***************************************
void mOGL::change_PolygonMode(void)
{
 glGetIntegerv(GL_POLYGON_MODE,&ogl_State[0]);
 // INFO: vypis nacitaneho stavu
 //fprintf(stderr,"ogl_State=%d\n",ogl_State[0]);

 if (ogl_State[0]==GL_LINE) glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
 else glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);

 SDL_Delay(200);
}

// zap/vyp casovo zavisleho pohybu objektov ********************************
void mOGL::change_PauseMotion(void)
{
 if (pause_motion==0) pause_motion=1;
 else pause_motion=0;

 SDL_Delay(200);
}

// zap/vyp vypisu helpu ****************************************************
void mOGL::change_HelpOnOff(void)
{
 if (help_onoff==0) help_onoff=1;
 else help_onoff=0;

 SDL_Delay(200);
}

// zap/vyp vykreslovania normal objektov ***********************************
void mOGL::change_NormalsOnOff(void)
{
 if (normals_onoff==0) normals_onoff=1;
 else normals_onoff=0;

 SDL_Delay(200);
}
