#ifndef _MOGL_H_
#define _MOGL_H_

// vlastne triedy
#include "mCamera.h"
#include "mText.h"
#include "mTexture.h"
#include "mShader.h"
#include "mTorus.h"

typedef struct _material { // struktura OpenGL materialu
 GLfloat amb[4],dif[4],spe[4],amb_dif[4],emi[4];
 GLfloat shi,transp,sillum[4];
};

typedef struct _light {    // struktura OpenGL svetla
 GLfloat amb[4],dif[4],spe[4];
 GLfloat pos[4],spot[3];
};

// trieda premennych a funkcii pre pracu s OpenGL
class mOGL {
public:
 mOGL();          // konstruktor
 virtual ~mOGL(); // destruktor

 GLint ViewPort[4]; // rozmery okna

 mCamera ogl_Camera[8]; // pouzite kamery

 bool gl_initialize(void); // inicializacia OpenGL
 int gl_draw(void);        // vykreslenie OpenGL sceny
 int gl_iddle(void);       // casovo zavisla uprava objektov, resp. sceny

 // zmena stavov OpenGL stavoveho stroja -----------------------------------
 void change_PolygonMode(void);   // zmena sposobu zobrazenia objektov
 void change_PauseMotion(void);   // zap/vyp casovo zavisleho pohybu objektov
 void change_HelpOnOff(void);     // zap/vyp vypisu helpu
 void change_NormalsOnOff(void);  // zap/vyp vykreslovania normal objektov
private:
 GLuint Cnum; // cislo pouzitej OpenGL kamery

 _material ogl_Material[8]; // pouzite materialy

 _light ogl_Light[8];       // pouzite svetla

 GLfloat StartInfoPos;                // pozicia vypisu start info
 void PrintStartInfo(char *infotext); // vypis info pocas startu aplikacie

 GLuint t_a,t_b,t_c; // pomocne premenne

 GLint ogl_State[2]; // pre zistovanie stavov OpenGL stavoveho stroja

 GLubyte lighting_onoff;  // indikator zap/vyp osvetlenia
 GLubyte help_onoff;      // indikator zap/vyp vypisu helpu
 GLubyte scene_draw_type; // pohlad na scenu
 GLubyte pause_motion;    // zastavenie casovo zavisleho pohybu objektov
 GLubyte normals_onoff;   // indikator zap/vyp vykreslovania normal

 GLfloat mtime; // cas

 // Shade Object
 tVec3 *SOvertex;     // pole vrcholov osvetleneho objektu
 tVec3 *SOnormal;     // pole normal osvetleneho objektu
 tVec3 *SOtangent;    // pole tangent-vektorov (U) osvetleneho objektu
 tVec3 *SObinormal;   // pole binormal-vektorov (V) osvetleneho objektu
 tVec2 *SOuv;         // pole UV texturovacich suradnic osvetleneho objektu
 GLuint SOvertex_num; // pocet vrcholov osvetleneho objektu
 GLuint SOdl_id;      // identifikator display listu osvetleneho objektu

 mShader torus_fp,point_vp,default_vp; // Fragment a Vertex programy

 tVec4 light1,light2,light3;    // poloha 3. svetiel
 tVec3 lcolor1,lcolor2,lcolor3; // farba 3. svetiel
 float angle1,angle2,angle3;    // uhol natocenia svetiel

 tVec3 rightv,upv; // pomocne vektory
 mat16 mv;         // pomocna matica

 float col[3];  // pomocna premenna pre ulozenie farby
 float sc;      // pomocna premenna pre ulozenie velkosti

 // dalsie pouzite triedy --------------------------------------------------
 mText ogl_Text;       // instancia triedy mText - praca s textom
 mTexture ogl_Texture; // instancia triedy mTexture - praca s texturou

 void RegCombinersON(float *color); // zapnutie Register kombinerov
 void RegCombinersOFF(void);        // vypnutie Register kombinerov
};

#endif
