// hlavickove subory *******************************************************
// systemove funkcie
#include <stdio.h>
#include <string.h>
// GL a GLEXT kniznica (OpenGL - pouzije sa subor /usr/lib/libGL.so
// pre gl.h aj pre glext.h, ostatne treba urobit v Makefile)
#include <GL/gl.h>
#include <GL/glext.h>
// funkcie SDL kniznice
#include <SDL/SDL.h>

// .h subor
#include "mShader.h"

// konstruktor *************************************************************
mShader::mShader()
{
 glActiveTextureARB=NULL;
 glMultiTexCoord2fARB=NULL;

 maxTexelUnits=1; // kazda karta ma aspon 1 texturovaciu jednotku
}

// destruktor **************************************************************
mShader::~mShader()
{
 // zrusenie objektu ShaderProgramu
 if (target==GL_FRAGMENT_PROGRAM_NV) glDeleteProgramsNV(1,&id);
 else glDeleteProgramsARB(1,&id);
}

// test ci karta podporuje rozsirenie **************************************
int mShader::CheckExtension(const char *ext_name)
{
 static char *extensions=NULL;

 // ziskanie retazca s podporovanymi rozsireniami grafickej karty
 if (!extensions) extensions=(char *) glGetString(GL_EXTENSIONS);

 // test, ci sa v retazci nachadza pozadovane rozsirenie
 if (!strstr(extensions,ext_name)) {
  fprintf(stderr,"OpenGL extension \"%s\" is not supported by current hardware!\n",ext_name);
  return(1);
 }
 return(0);
}

// inicializacia rozsireni *************************************************
int mShader::Ext_init()
{
 // ziskanie poctu texturovacich jednotiek
 glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB,&maxTexelUnits);
 // prevzatie adries rozsirenych funkcii
 glActiveTextureARB=(PFNGLACTIVETEXTUREARBPROC) SDL_GL_GetProcAddress("glActiveTextureARB");
 glMultiTexCoord2fARB=(PFNGLMULTITEXCOORD2FARBPROC) SDL_GL_GetProcAddress("glMultiTexCoord2fARB");

 return(0);
}

// nahratie Shader Programu ************************************************
int mShader::PRGload(const char *src)
{
 char *sh_code;
 // otvorenie suboru s kodom ShaderProgramu
 FILE *file=fopen(src,"r");
 // nacitanie obsahu suboru do premennej sh_code
 if (file) {
  fseek(file,0,SEEK_END);
  int size=ftell(file);
  sh_code=new char[size+1];
  fseek(file,0,SEEK_SET);
  fread(sh_code,1,size,file);
  sh_code[size]='\0';
  fclose(file);
 } else {
    target=0; id=0;
    fprintf(stderr,"Error open \"%s\" file!\n",src);
    return(1);
   }

 // vytvorenie objektu ShaderProgramu
 error=-1;
 if (!strncmp(sh_code,"!!ARBvp1.0",10)) { // test ci je to ARB VertexProgram
  target=GL_VERTEX_PROGRAM_ARB;
  // vytvorenie objektu ShaderProgramu
  glGenProgramsARB(1,&id);
  glBindProgramARB(target,id);
  // umiestnenie kodu do objektu ShaderProgramu
  glProgramStringARB(target,GL_PROGRAM_FORMAT_ASCII_ARB,strlen(sh_code),sh_code);
  glGetIntegerv(GL_PROGRAM_ERROR_POSITION_ARB,&error);
 } else if (!strncmp(sh_code,"!!ARBfp1.0",10)) { // test ci je to ARB FragmentProgram
         target=GL_FRAGMENT_PROGRAM_ARB;
         // vytvorenie objektu ShaderProgramu
         glGenProgramsARB(1,&id);
         glBindProgramARB(target,id);
         // umiestnenie kodu do objektu ShaderProgramu
         glProgramStringARB(target,GL_PROGRAM_FORMAT_ASCII_ARB,strlen(sh_code),sh_code);
         glGetIntegerv(GL_PROGRAM_ERROR_POSITION_ARB,&error);
        } else if (!strncmp(sh_code,"!!FP1.0",7)) { // test ci je to NV FragmentProgram
                target=GL_FRAGMENT_PROGRAM_NV;
                // vytvorenie objektu ShaderProgramu
                glGenProgramsNV(1,&id);
                glBindProgramNV(target,id);
                // umiestnenie kodu do objektu ShaderProgramu
                glLoadProgramNV(target,id,strlen(sh_code),(GLubyte *) sh_code);
                glGetIntegerv(GL_PROGRAM_ERROR_POSITION_NV,&error);
               } else { // nie je to ziaden z podporovanych ShaderProgramov
                  target=0; id=0;
                  fprintf(stderr,"Unknown program header of \"%s\" file!\n",src);
                  delete sh_code;
                  return(1);
                 }

 // ak nastala chyba, nasleduje vypis riadku ShaderProgramu s najdenou chybou
 if (error!=-1) {
  int line=0;      // cislo riadku
  char *s=sh_code; // pomocna premenna
  while (error-- && *s) if (*s++ == '\n') line++;
  while (s >= sh_code && *s != '\n') s--;
  fprintf(stderr,"Program error at line %d:\n\"%s\"\n",line,s);
 }

 delete sh_code;
 return(0);
}

// zapnutie ShaderProgramu *************************************************
void mShader::PRGenable()
{
 glEnable(target);
}

// vypnutie ShaderProgramu *************************************************
void mShader::PRGdisable()
{
 glDisable(target);
}

// pripojenie ShaderProgramu ***********************************************
void mShader::PRGbind()
{
 if (target==GL_FRAGMENT_PROGRAM_NV) glBindProgramNV(target,id);
 else glBindProgramARB(target,id);
}

// vlozenie Environment parametra do ARB ShaderProgramu ********************
void mShader::PRGenv_param(GLuint index,const tVec4 &p)
{
 glProgramEnvParameter4fvARB(target,index,p);
}

// vlozenie Local parametra do ARB ShaderProgramu **************************
void mShader::PRGlocal_param(GLuint index,const tVec4 &p)
{
 glProgramLocalParameter4fvARB(target,index,p);
}

// vlozenie Named parametra do NV ShaderProgramu ***************************
void mShader::PRGnamed_param(const char *name,const tVec4 &p)
{
 glProgramNamedParameter4fvNV(id,strlen(name),(GLubyte *) name,p);
}
