// hlavickove subory *******************************************************
// systemove funkcie
//#include <string.h>
// GL, GLU a GLEXT kniznice
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glext.h>
// funkcie SDL kniznice
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

// .h subor
#include "mTexture.h"

// SDL nacitanie suborov s texturami
SDL_Surface *TextureImage;

// konstruktor *************************************************************
mTexture::mTexture()
{
 texture_id[0]=0;
 texture_count=0;

 crosshair_tex_id=0;

 TextureImage=NULL;
}

// destruktor **************************************************************
mTexture::~mTexture()
{
 for (t_a=0;t_a<texture_count;t_a++) glDeleteTextures(1,&texture_id[t_a]);
 glDeleteTextures(1,&crosshair_tex_id);
}

// nahratie textury ********************************************************
int mTexture::LoadTexture(GLuint id,char *myfile,GLuint mode)
{
 TextureImage=IMG_Load(myfile);

 // INFO: identifikator a nazov suboru textury
 fprintf(stderr,"textura: %i %s\n",id,myfile);

 // nahratie suboru s obrazkom
 if (TextureImage!=NULL) {
  glGenTextures(1,&texture_id[id]);
  // pripojenie textury
  glBindTexture(GL_TEXTURE_2D,texture_id[id]);

  // nastavenie filteringu
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  if (mode&T_MIPMAP) {
   if (mode&T_TRILINEAR) {
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
   } else {
      glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
     }
  }

  // nastavenie opakovania sa textury
  if (mode&T_CLAMP) {
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
  } else if (mode&T_CLAMP_TO_EDGE) {
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
         }
         else {
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
         }

  // vygenerovanie textury
  if (mode&T_MIPMAP) {
   // vygenerovanie MIPMAP textury
   gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,TextureImage->w,TextureImage->h,
                     GL_BGR,GL_UNSIGNED_BYTE,TextureImage->pixels);
  } else {
     // vygenerovanie standardnej 2D textury
     glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,TextureImage->w,TextureImage->h,
                  0,GL_BGR,GL_UNSIGNED_BYTE,TextureImage->pixels);
    }

  texture_count++;
 }
 else return(1);

 // uvolnenie pouzitej pamate
 if (TextureImage) SDL_FreeSurface(TextureImage);
 TextureImage=NULL;

 return(0);
}

// nahratie 1D textury *****************************************************
int mTexture::Load1DTexture(GLuint id,char *myfile,GLuint mode)
{
 TextureImage=IMG_Load(myfile);

 // INFO: identifikator a nazov suboru textury
 fprintf(stderr,"textura: %i %s\n",id,myfile);

 // nahratie suboru s obrazkom
 if (TextureImage!=NULL) {
  glGenTextures(1,&texture_id[id]);
  // pripojenie textury
  glBindTexture(GL_TEXTURE_1D,texture_id[id]);

  // nastavenie filteringu
  glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  if (mode&T_MIPMAP) {
   if (mode&T_TRILINEAR) {
    glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
   } else {
      glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
     }
  }

  // nastavenie opakovania sa textury
  if (mode&T_CLAMP) {
   glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_WRAP_S,GL_CLAMP);
  } else if (mode&T_CLAMP_TO_EDGE) {
          glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
         }
         else {
          glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_WRAP_S,GL_REPEAT);
         }

  // vygenerovanie textury
  if (mode&T_MIPMAP) {
   // vygenerovanie MIPMAP textury
   gluBuild1DMipmaps(GL_TEXTURE_1D,GL_RGB,TextureImage->w,
                     GL_BGR,GL_UNSIGNED_BYTE,TextureImage->pixels);
  } else {
     // vygenerovanie standardnej 1D textury
     glTexImage1D(GL_TEXTURE_1D,0,GL_RGB,TextureImage->w,0,
                  GL_BGR,GL_UNSIGNED_BYTE,TextureImage->pixels);
    }

  texture_count++;
 }
 else return(1);

 // uvolnenie pouzitej pamate
 if (TextureImage) SDL_FreeSurface(TextureImage);
 TextureImage=NULL;

 return(0);
}

// nahratie bump textury ***************************************************
int mTexture::LoadBumpTexture(GLuint id1,GLuint id2,char *myfile,GLuint mode)
{
 TextureImage=IMG_Load(myfile);

 // INFO: identifikator a nazov suboru textury
 fprintf(stderr,"bump textura: %i %s\n",id1,myfile);

 // nahratie suboru s obrazkom
 if (TextureImage!=NULL) {
  glPixelTransferf(GL_RED_SCALE,  0.5f); // 50% intenzita
  glPixelTransferf(GL_GREEN_SCALE,0.5f);
  glPixelTransferf(GL_BLUE_SCALE, 0.5f);

  glGenTextures(1,&texture_id[id1]);
  // pripojenie textury
  glBindTexture(GL_TEXTURE_2D,texture_id[id1]);

  // nastavenie filteringu
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  if (mode&T_MIPMAP) {
   if (mode&T_TRILINEAR) {
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
   } else {
      glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
     }
  }

  // nastavenie opakovania sa textury
  if (mode&T_CLAMP) {
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
  } else if (mode&T_CLAMP_TO_EDGE) {
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
         }
         else {
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
         }

  // vygenerovanie textury
  if (mode&T_MIPMAP) {
   // vygenerovanie MIPMAP textury
   gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB8,TextureImage->w,TextureImage->h,
                     GL_BGR,GL_UNSIGNED_BYTE,TextureImage->pixels);
  } else {
     // vygenerovanie standardnej 2D textury
     glTexImage2D(GL_TEXTURE_2D,0,GL_RGB8,TextureImage->w,TextureImage->h,
                  0,GL_BGR,GL_UNSIGNED_BYTE,TextureImage->pixels);
    }

  // vytvorenie invertovanej textury
  td=(unsigned char *) TextureImage->pixels;
  for (ti=0;ti<3*TextureImage->w*TextureImage->h;ti++) td[ti]=255-td[ti];

  glGenTextures(1,&texture_id[id2]);
  // pripojenie textury
  glBindTexture(GL_TEXTURE_2D,texture_id[id2]);

  // nastavenie filteringu
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  if (mode&T_MIPMAP) {
   if (mode&T_TRILINEAR) {
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
   } else {
      glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
     }
  }

  // nastavenie opakovania sa textury
  if (mode&T_CLAMP) {
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
  } else if (mode&T_CLAMP_TO_EDGE) {
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
         }
         else {
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
          glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
         }

  // vygenerovanie textury
  if (mode&T_MIPMAP) {
   // vygenerovanie MIPMAP textury
   gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB8,TextureImage->w,TextureImage->h,
                     GL_BGR,GL_UNSIGNED_BYTE,td);
  } else {
     // vygenerovanie standardnej 2D textury
     glTexImage2D(GL_TEXTURE_2D,0,GL_RGB8,TextureImage->w,TextureImage->h,
                  0,GL_BGR,GL_UNSIGNED_BYTE,td);
    }

  td=NULL;

  texture_count+=2;

  glPixelTransferf(GL_RED_SCALE,  1.0f); // 100% intenzita
  glPixelTransferf(GL_GREEN_SCALE,1.0f);
  glPixelTransferf(GL_BLUE_SCALE, 1.0f);
 }
 else return(1);

 // uvolnenie pouzitej pamate
 if (TextureImage) SDL_FreeSurface(TextureImage);
 TextureImage=NULL;

 return(0);
}

// nahratie Cube Map textury ***********************************************
int mTexture::LoadCubeMapTexture(char *mf1,char *mf2,char *mf3,
                                 char *mf4,char *mf5,char *mf6,GLuint mode)
{
 glGenTextures(1,&cubemap_id);
 // pripojenie textury
 glBindTexture(GL_TEXTURE_CUBE_MAP_ARB,cubemap_id);
 // nastavenie filteringu
 glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
 glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
 // nastavenie opakovania sa textury
 if (mode&T_CLAMP_TO_EDGE) {
  glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
 } else {
    glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_WRAP_S,GL_REPEAT);
    glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB,GL_TEXTURE_WRAP_T,GL_REPEAT);
   }
 // vlozenie obrazkov do textury
 for (t_a=0;t_a<6;t_a++) {
  switch(t_a) {
   case 0: TextureImage=IMG_Load(mf1); 
           fprintf(stderr,"textura: %i %s\n",t_a,mf1); // INFO: identifikator a nazov suboru textury
           glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB,0,GL_RGB,
                        TextureImage->w,TextureImage->h,0,
                        GL_BGR,GL_UNSIGNED_BYTE, TextureImage->pixels);
   break;
   case 1: TextureImage=IMG_Load(mf2);
           fprintf(stderr,"textura: %i %s\n",t_a,mf2); // INFO: identifikator a nazov suboru textury
           glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB,0,GL_RGB,
                        TextureImage->w,TextureImage->h,0,
                        GL_BGR,GL_UNSIGNED_BYTE, TextureImage->pixels);
   break;
   case 2: TextureImage=IMG_Load(mf3);
           fprintf(stderr,"textura: %i %s\n",t_a,mf3); // INFO: identifikator a nazov suboru textury
           glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB,0,GL_RGB,
                        TextureImage->w,TextureImage->h,0,
                        GL_BGR,GL_UNSIGNED_BYTE, TextureImage->pixels);
   break;
   case 3: TextureImage=IMG_Load(mf4);
           fprintf(stderr,"textura: %i %s\n",t_a,mf4); // INFO: identifikator a nazov suboru textury
           glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB,0,GL_RGB,
                        TextureImage->w,TextureImage->h,0,
                        GL_BGR,GL_UNSIGNED_BYTE, TextureImage->pixels);
   break;
   case 4: TextureImage=IMG_Load(mf5);
           fprintf(stderr,"textura: %i %s\n",t_a,mf5); // INFO: identifikator a nazov suboru textury
           glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB,0,GL_RGB,
                        TextureImage->w,TextureImage->h,0,
                        GL_BGR,GL_UNSIGNED_BYTE, TextureImage->pixels);
   break;
   case 5: TextureImage=IMG_Load(mf6);
           fprintf(stderr,"textura: %i %s\n",t_a,mf6); // INFO: identifikator a nazov suboru textury
           glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB,0,GL_RGB,
                        TextureImage->w,TextureImage->h,0,
                        GL_BGR,GL_UNSIGNED_BYTE, TextureImage->pixels);
   break;
  };

  texture_count++;
 }

 // uvolnenie pouzitej pamate
 if (TextureImage) SDL_FreeSurface(TextureImage);
 TextureImage=NULL;

 return(0);
}

// zrusenie OpenGL textury *************************************************
void mTexture::FreeTexture(GLuint id)
{
 glDeleteTextures(1,&texture_id[id]);
}

// nahratie textury zameriavacieho kriza ***********************************
int mTexture::LoadCrosshairTexture(char *myfile)
{
 // nahratie suboru s obrazkom
 if ((TextureImage=IMG_Load(myfile))) {
  glGenTextures(1,&crosshair_tex_id);
  // pripojenie textury
  glBindTexture(GL_TEXTURE_2D,crosshair_tex_id);
  // nastavenie filteringu
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  // nastavenie opakovania sa textury
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
  // vygenerovanie textury
  glTexImage2D(GL_TEXTURE_2D,0,3,TextureImage->w,TextureImage->h,
               0,GL_BGR,GL_UNSIGNED_BYTE,TextureImage->pixels);
 }
 else return(1);

 // uvolnenie pouzitej pamate
 if (TextureImage) SDL_FreeSurface(TextureImage);
 TextureImage=NULL;

 return(0);
}

// vykreslenie zameriavacieho kriza ****************************************
int mTexture::DrawCrosshair(GLfloat w,GLfloat h)
{
 // ulozenie potrebnych atributov
 glPushAttrib(GL_LIGHTING_BIT|GL_DEPTH_BUFFER_BIT|GL_TEXTURE_BIT);
 glDisable(GL_LIGHTING);   // vypnutie osvetlenia
 glDisable(GL_DEPTH_TEST); // vypnutie testu hlbky
 glEnable(GL_TEXTURE_2D);  // zapnutie texturovania

 glBlendFunc(GL_ONE,GL_ONE); // zapnutie priesvitnosti
 glEnable(GL_BLEND);

//// glMatrixMode(GL_PROJECTION); // uprava projekcnej matice
//// glPushMatrix();
//// glLoadIdentity();
//// glOrtho(0.0f,w,0.0f,h,-1.0f,1.0f); // pravouhly suradnicovy system
//// glMatrixMode(GL_MODELVIEW);  // uprava pohladovej matice
 glPushMatrix();
 glLoadIdentity();

 // pripojenie textury zameriavacieho kriza
 glBindTexture(GL_TEXTURE_2D,crosshair_tex_id);
 // zameriavaci kriz na stred okna
//// glTranslatef(w/2.0f,h/2.0f,-0.1f);
 glTranslatef(0.0f,0.0f,-0.1f);
 // stvoruholnik
 glBegin(GL_QUADS);
  glTexCoord2f(0.0f,0.0f); glVertex3f(-0.0075f,-0.0075f,0.0f);
  glTexCoord2f(1.0f,0.0f); glVertex3f( 0.0075f,-0.0075f,0.0f);
  glTexCoord2f(1.0f,1.0f); glVertex3f( 0.0075f, 0.0075f,0.0f);
  glTexCoord2f(0.0f,1.0f); glVertex3f(-0.0075f, 0.0075f,0.0f);
 //// glTexCoord2f(0.0f,0.0f); glVertex3f(-32.0f,-32.0f,0.0f);
 //// glTexCoord2f(1.0f,0.0f); glVertex3f( 32.0f,-32.0f,0.0f);
 //// glTexCoord2f(1.0f,1.0f); glVertex3f( 32.0f, 32.0f,0.0f);
 //// glTexCoord2f(0.0f,1.0f); glVertex3f(-32.0f, 32.0f,0.0f);
 glEnd();

 glPopMatrix(); // pohladova
//// glMatrixMode(GL_PROJECTION);
//// glPopMatrix(); // projekcna
//// glMatrixMode(GL_MODELVIEW); // aktualna je vzdy pohladova

 glDisable(GL_BLEND); // vypnutie priesvitnosti

 glPopAttrib(); // vyber ulozenych atributov

 return(0);
}
